#!/usr/bin/env python

"""
AgentStatusWatcher test
"""
import os
import threading
import unittest
from WMQuality.TestInitCouchApp import TestInitCouchApp as TestInit

from WMComponent.AgentStatusWatcher.AgentStatusPoller import AgentStatusPoller


class AgentStatusWatcher_t(unittest.TestCase):
    """
    TestCase for AgentStatustWatcher module
    """

    def setUp(self):
        """
        setup for test.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.reqmonDBName = "wmstat_t"
        self.localDBName = "wmstat_t_local"
        self.testInit.setupCouch(self.reqmonDBName, "WMStatsAgent")
        self.testInit.setupCouch(self.localDBName, "WMStatsAgent")
        self.testDir = self.testInit.generateWorkDir()
        return

    def tearDown(self):
        """
        Database deletion
        """
        threading.currentThread()

        self.testInit.delWorkDir()
        self.testInit.tearDownCouch()
        self.testInit.clearDatabase()
        return

    def getConfig(self):
        """
        _createConfig_

        General config file
        """

        couchURL = self.testInit.couchUrl
        config = self.testInit.getConfiguration()
        self.testInit.generateWorkDir(config)

        config.section_("Agent")
        config.Agent.hostName = "localhost"
        config.Agent.contact = "test@test.com"
        config.Agent.teamName = "testTeam"
        config.Agent.agentName = "testAgentName"
        config.Agent.useMsgService = False
        config.Agent.useTrigger = False
        config.Agent.useHeartbeat = False

        config.component_("AgentStatusWatcher")
        config.AgentStatusWatcher.namespace = "WMComponent.AgentStatusWatcher.AgentStatusWatcher"
        config.AgentStatusWatcher.jsonFile = "/dev/null"

        config.component_("AnalyticsDataCollector")
        config.AnalyticsDataCollector.namespace = "WMComponent.AnalyticsDataCollector.AnalyticsDataCollector"
        config.AnalyticsDataCollector.componentDir = config.General.workDir + "/AnalyticsDataCollector"
        config.AnalyticsDataCollector.localQueueURL = "%s/%s" % (couchURL, "workqueue")
        config.AnalyticsDataCollector.summaryLevel = "task"

        return config

    def testAgentStatusPoller(self):
        """
        Tests the components, as in sees if they load.
        Otherwise does nothing.
        """
        threading.currentThread()
        config = self.getConfig()
        # change between RPM and Docker based tests
        print(f"WMA_DEPLOY_DIR: {os.environ.get('WMA_DEPLOY_DIR')}")
        print(f"WMCORE_ROOT: {os.environ.get('WMCORE_ROOT')}")
        if os.environ.get('WMA_DEPLOY_DIR'):
            ## check for Alma9 docker
            print(f"Found WMA_DEPLOY_DIR set to: {os.environ.get('WMA_DEPLOY_DIR')}")
        elif os.environ.get('TEST_DIR'):
            # check for CC7 docker
            print(f"Found TEST_DIR set to: {os.environ.get('TEST_DIR')}")
            os.environ["WMA_DEPLOY_DIR"] = os.path.join(os.environ.get('TEST_DIR'), "WMCore")
        elif os.environ.get('WMCORE_ROOT'):
            # check for CC7 RPM
            print(f"Found WMCORE_ROOT set to: {os.environ.get('WMCORE_ROOT')}")
            #os.environ["WMA_DEPLOY_DIR"] = os.path.join(os.environ.get('WMCORE_ROOT'), "install")
            os.environ["WMA_DEPLOY_DIR"] = os.environ.get('WMCORE_ROOT')
        else:
            # last fallback path
            fallbackDir = "/home/cmsbld/WMCore/"
            print("Did not find any of the expected env vars: WMA_DEPLOY_DIR, TEST_DIR, WMCORE_ROOT")
            print(f"Setting WMA_DEPLOY_DIR environment variable to: {fallbackDir}")
            os.environ["WMA_DEPLOY_DIR"] = os.path.join(fallbackDir, "install")

        watcher = AgentStatusPoller(config)
        # Note:
        # Should call: watcher.setup() and watcher.algorithm()
        # for a complete poller test.
        # Just testing proxy/certificate lifetimes for now.

        # Start with ok status
        agInfo = {'status':'ok'}

        # Check service certificate and proxy lifetimes
        watcher.checkCredLifetime(agInfo, "proxy")
        watcher.checkCredLifetime(agInfo, "certificate")
        print(f"Agent Info:\n{agInfo}")

if __name__ == '__main__':
    unittest.main()



"""
_CountFinishedSubscriptionsByTask_

Oracle implementation of CountFinishedSubscriptionsByTask
"""
from WMCore.WMBS.MySQL.Subscriptions.CountFinishedSubscriptionsByTask \
     import CountFinishedSubscriptionsByTask as MySQLCountFinishedSubscriptionsByTask

class CountFinishedSubscriptionsByTask(MySQLCountFinishedSubscriptionsByTask):
    pass



#!/usr/bin/env python
"""
_UpdateAlgoDatasetAssoc_

Oracle implementation of DBSBuffer.UpdateAlgoDatasetAssoc
"""




from WMComponent.DBS3Buffer.MySQL.UpdateAlgoDatasetAssoc import UpdateAlgoDatasetAssoc as MySQLUpdateAlgoDatasetAssoc

class UpdateAlgoDatasetAssoc(MySQLUpdateAlgoDatasetAssoc):
    """
    _UpdateAlgoDatasetAssoc_

    Update the in_dbs column for a particular algo/dataset association.
    """
    pass



#!/usr/bin/env python
# encoding: utf-8
"""
CollectionTypes.py

Created by Dave Evans on 2010-03-05.
Copyright (c) 2010 Fermilab. All rights reserved.
"""


GenericCollection = "ACDC.CollectionTypes.GenericCollection"
DataCollection = "ACDC.CollectionTypes.DataCollection"



from __future__ import (division, print_function)
from builtins import object


class MockMemoryCacheStruct(object):
    """
    Version of Services/PhEDEx intended to be used with mock or unittest.mock
    """

    def __init__(self, *args, **kwargs):
        print("Using MockMemoryCache")

    def getData(self, noFail=True):
        return {}



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
_WMException_t_

General test for WMException, intended to simulate code

- run from a py2 interpreter
- without any modernization to strings, such as
  - no `from builtins import str`
  - no `from builtins import bytes`

"""
from __future__ import print_function, division

import logging
import unittest
from WMCore.WMException import WMException


class WMExceptionTest(unittest.TestCase):
    """
    A test of a generic exception class
    """
    def setUp(self):
        """
        setup log file output.
        """
        logging.basicConfig(level=logging.NOTSET,
                            format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                            datefmt='%m-%d %H:%M',
                            filename='%s.log' % __file__,
                            filemode='w')

        self.logger = logging.getLogger('WMExceptionTest')

        self.test_data = {
            'key-ascii': 'value-1',       # unicode (ascii): unicode (ascii)
            'key-ascii-b': '√@ʟυℯ-1',     # unicode (ascii): unicode (non-ascii)
            'key-ascii-c': u'√@ʟυℯ-1',    # unicode (ascii): unicode (non-ascii)
            'key-ascii-d': b'√@ʟυℯ-1',    # unicode (ascii): bytes (of non-ascii)
            'ḱℯƴ-unicode-a': 'ṽ@łυ℮-2',   # unicode (non-ascii): unicode (non-ascii)
            u'ḱℯƴ-unicode-b': 'ṽ@łυ℮-2',  # unicode (non-ascii): unicode (non-ascii)
            b'ḱℯƴ-unicode-c': 'ṽ@łυ℮-2',  # bytes (of non-ascii): unicode (non-ascii)
            'ḱℯƴ-unicode-d': 'value-\x95',  # unicode (of non-ascii): unicode (invalid byte)
            'key-\x95': 'ṽ@łυ℮-2',  # unicode (invalid byte): unicode (non-ascii)
            'key3': 3.14159,
            # This would break WMException, but should not happen
            # 'key4': {
            #     b'ḱℯƴ-unicode-c': 'ṽ@łυ℮-2',  # bytes (of unicode): unicode
            # }
            }

    def tearDown(self):
        """
        nothing to tear down
        """

        pass

    def testExceptionUnicode1(self):
        """
        create an exception with non-ascii chars in message and test WMException constructor
        """

        exception = WMException("an exception message with nr. 100 and some non-ascii characters: ₩♏ℭ☺яε", 100, **self.test_data)
        self.logger.debug("XML version of exception: %s", exception.xml())
        self.logger.debug("String version of exception: %s", unicode(exception))
        self.logger.debug("String version of exception: %s", str(exception))
        self.logger.debug("exception.__str__(): %s", type(exception.__str__()))  # <class 'future.types.newbytes.newbytes'>
        self.logger.debug("str(exception): %s", type(str(exception)))  # <class 'future.types.newbytes.newbytes'>
        self.logger.debug("unicode(exception): %s", type(unicode(exception)))  # <class 'future.types.newstr.newstr'>


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

"""
Stuff to help with debugging from https://pymotw.com/3/sys/tracing.html

Use like this:

        import sys
        from Utils.Tracing import trace_calls_and_returns, dont_trace

        sys.settrace(trace_calls_and_returns)
        # Do the things you want to trace
        sys.settrace(dont_trace)

"""


def trace_calls(frame, event, arg, to_be_traced=None):
    if event != 'call':
        return
    co = frame.f_code
    func_name = co.co_name
    if func_name == 'write':
        return  # Ignore write() calls from printing
    line_no = frame.f_lineno
    filename = co.co_filename
    print('* Call to {} on line {} of {}'.format(
        func_name, line_no, filename))
    if func_name in to_be_traced:
        # Trace into this function
        return trace_lines
    return


def trace_calls_and_returns(frame, event, arg):
    co = frame.f_code
    func_name = co.co_name
    if func_name == 'write':
        return  # Ignore write() calls from printing
    line_no = frame.f_lineno
    filename = co.co_filename
    if event == 'call':
        print('Call to {} on line {} of {}'.format(func_name, line_no, filename))
        return trace_calls_and_returns
    elif event == 'return':
        print(' {} => {} ({})'.format(func_name, arg, filename))
    return


def trace_lines(frame, event, arg):
    if event != 'line':
        return
    co = frame.f_code
    func_name = co.co_name
    line_no = frame.f_lineno
    print('*  {} line {}'.format(func_name, line_no))


def dont_trace(frame, event, arg):
    return



#!/usr/bin/env python
"""
Test of assertContentsEqual for ExtendedUnitTestCase
"""

import copy
import unittest

from Utils.ExtendedUnitTestCase import ExtendedUnitTestCase


class ExtendedUnitTestCaseTest(ExtendedUnitTestCase):
    """
    Test of assertContentsEqual for ExtendedUnitTestCase
    """

    def testNestedListComparison(self):
        """
        Test the nested comparison method. Deeply nested lists are compared by content, not order
         (assertItemsEqual in python 2.7, assertCountEqual in python3)
        """

        initialList = [{'a': [1, 2, 3, 4], 'b': ['a', 'b', 'c', 'd'], 'c': 'd'},
                       {'e': {'c': [1, 2, 3]}}]
        copyList = copy.deepcopy(initialList)
        sameList = [{'e': {'c': [3, 2, 1]}},
                    {'a': [3, 1, 2, 4], 'b': ['b', 'a', 'c', 'd'], 'c': 'd'}]
        diffList1 = [{'a': [1, 2, 3, 5], 'b': ['a', 'b', 'c', 'e'], 'c': 'd'}, {'e': {'c': [1, 2, 3]}}]
        diffList2 = [{'a': [1, 2, 3, 4], 'b': ['a', 'b', 'c', 'd'], 'c': 'd'}, {'e': {'c': [3, 1, 3]}}]

        self.assertNotEqual(initialList, sameList)
        self.assertContentsEqual(initialList, sameList)
        self.assertNotEqual(initialList, sameList)  # Make sure they are still not equal (were not modified)
        self.assertEqual(initialList, copyList)

        with self.assertRaises(AssertionError):
            self.assertContentsEqual(initialList, diffList1)
        with self.assertRaises(AssertionError):
            self.assertContentsEqual(initialList, diffList2)

    def testNestedDictComparison(self):
        """
        Test the nested comparison method. Deeply nested lists are compared by content, not order
         (assertItemsEqual in python 2.7, assertCountEqual in python3)
        """

        initialDict = {'a': [1, 2, 3, 4],
                       'b': ['a', 'b', 'c', 'd'],
                       'c': 'd',
                       'e': {'c': [1, 2, 3]}}
        copyDict = copy.deepcopy(initialDict)
        sameDict = {'a': [3, 1, 4, 2],
                    'b': ['a', 'b', 'd', 'c'],
                    'c': 'd',
                    'e': {'c': [3, 2, 1]}}
        diffDict1 = {'a': [3, 1, 4, 2],
                     'b': ['a', 'b', 'd', 'c'],
                     'c': 'd',
                     'e': {'c': 'f'}}
        diffDict2 = {'a': [1, 2, 3, 4],
                     'b': ['a', 'b', 'c', 'd'],
                     'c': 'd',
                     'e': {'c': [3, 2, 3]}}

        self.assertNotEqual(initialDict, sameDict)
        self.assertContentsEqual(initialDict, sameDict)
        self.assertNotEqual(initialDict, sameDict)  # Make sure they are still not equal (were not modified)
        self.assertEqual(initialDict, copyDict)

        with self.assertRaises(AssertionError):
            self.assertContentsEqual(initialDict, diffDict1)
        with self.assertRaises(AssertionError):
            self.assertContentsEqual(initialDict, diffDict2)

    def testWrongTypes(self):
        """
        Test that comparison fails with different types
        """

        d = {}
        l = []
        s = ()

        with self.assertRaises(AssertionError):
            self.assertContentsEqual(d, l)
        with self.assertRaises(AssertionError):
            self.assertContentsEqual(l, s)


if __name__ == '__main__':
    unittest.main()



"""
_EventAwareLumiByWork_t_

Lumi based splitting test with awareness of events per lumi, using the DataStructs classes.
It must pass the same tests as the LumiBased algorithm, plus
specific ones for this algorithm.
See WMCore/WMBS/JobSplitting/ for the WMBS (SQL database) version.
"""

from __future__ import division, print_function

from builtins import range
import logging
import unittest
from collections import Counter

from Utils.PythonVersion import PY3

from WMCore.DataStructs.File import File
from WMCore.DataStructs.Fileset import Fileset
from WMCore.DataStructs.LumiList import LumiList
from WMCore.DataStructs.Run import Run
from WMCore.DataStructs.Subscription import Subscription
from WMCore.DataStructs.Workflow import Workflow
from WMCore.JobSplitting.SplitterFactory import SplitterFactory
from WMCore.Services.UUIDLib import makeUUID


class EventAwareLumiByWorkTest(unittest.TestCase):
    """
    _EventAwareLumiByWorkTest_

    Test event based job splitting.
    """

    def setUp(self):
        """
        _setUp_

        Create two subscriptions: One that contains a single file and one that
        contains multiple files.
        """

        self.testWorkflow = Workflow()
        self.performanceParams = {'timePerEvent': 12, 'memoryRequirement': 2300, 'sizePerEvent': 400}

        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)

        if PY3:
            self.assertItemsEqual = self.assertCountEqual

        return

    def createSubscription(self, nFiles, lumisPerFile, twoSites=False, nEventsPerFile=100):
        """
        _createSubscription_

        Create a subscription for testing
        """

        baseName = makeUUID()

        testFileset = Fileset(name=baseName)
        for i in range(nFiles):
            newFile = self.createFile('%s_%i' % (baseName, i), nEventsPerFile, i, lumisPerFile, 'blenheim')
            testFileset.addFile(newFile)
        if twoSites:
            for i in range(nFiles):
                newFile = self.createFile('%s_%i_2' % (baseName, i), nEventsPerFile, nFiles+i, lumisPerFile, 'malpaquet')
                testFileset.addFile(newFile)

        testSubscription = Subscription(fileset=testFileset, workflow=self.testWorkflow,
                                        split_algo="EventAwareLumiByWork", type="Processing")

        return testSubscription

    @staticmethod
    def createFile(lfn, events, run, lumis, location):
        """
        _createFile_

        Create a file for testing
        """
        newFile = File(lfn=lfn, size=1000, events=events)
        lumiList = []
        for lumi in range(lumis):
            lumiList.append((run * lumis) + lumi)
        newFile.addRun(Run(run, *lumiList))
        newFile.setLocation(location)
        return newFile

    def testFileSplitting(self):
        """
        _testFileSplitting_

        Test that things work if we split files between jobs
        """
        splitter = SplitterFactory()

        oneSetSubscription = self.createSubscription(nFiles=10, lumisPerFile=1)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=oneSetSubscription)

        jobGroups = jobFactory(halt_job_on_file_boundaries=True, events_per_job=100, performance=self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 10)
        for job in jobGroups[0].jobs:
            self.assertTrue(len(job['input_files']), 1)

        twoLumiFiles = self.createSubscription(nFiles=5, lumisPerFile=2)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=twoLumiFiles)
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, events_per_job=50, performance=self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 10)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job['input_files']), 1)

        wholeLumiFiles = self.createSubscription(nFiles=5, lumisPerFile=3)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=wholeLumiFiles)
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, events_per_job=67, performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        # 10 because we split on run boundaries
        self.assertEqual(len(jobGroups[0].jobs), 10)
        jobList = jobGroups[0].jobs
        for job in jobList:
            # Have should have one file, half two
            self.assertTrue(len(job['input_files']) in [1, 2])

        jobLumiList = [jobList[i]['mask'].getRunAndLumis() for i in range(0, 10)]
        correctJobLumiList = [{0: [[0, 1]]}, {0: [[2, 2]]},
                              {1: [[3, 4]]}, {1: [[5, 5]]},
                              {4: [[12, 13]]}, {4: [[14, 14]]}
                             ]

        for lumiList in correctJobLumiList:
            self.assertIn(lumiList, jobLumiList)

        # Do it with multiple sites
        twoSiteSubscription = self.createSubscription(nFiles=5, lumisPerFile=2, twoSites=True)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=twoSiteSubscription)
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, events_per_job=50, performance=self.performanceParams)
        self.assertEqual(len(jobGroups), 2)
        self.assertEqual(len(jobGroups[0].jobs), 10)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job['input_files']), 1)

    def testNoRunNoFileSplitting(self):
        """
        _testNoRunNoFileSplitting_

        Test the splitting algorithm in the odder fringe
        cases that might be required.
        """
        splitter = SplitterFactory()
        testSubscription = self.createSubscription(nFiles=5, lumisPerFile=5, twoSites=False)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)
        jobGroups = jobFactory(halt_job_on_file_boundaries=False, splitOnRun=False, events_per_job=60,
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 9)
        self.enforceLimits(filesPerJob=2, jobsPerFile=3)

        # Assert that this works differently with file splitting on and run splitting on
        testSubscription = self.createSubscription(nFiles=5, lumisPerFile=5, twoSites=False)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, splitOnRun=True, events_per_job=60,
                               performance=self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 10)
        self.enforceLimits(filesPerJob=1, jobsPerFile=2, runsPerJob=1)

        # Test total_events limit. (The algorithm cuts off after the lumi that
        # brings the total average event count over -or equal to- total_events.)
        testSubscription = self.createSubscription(nFiles=5, lumisPerFile=5, twoSites=False)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)

        jobGroups = jobFactory(halt_job_on_file_boundaries=False, splitOnRun=False, events_per_job=60, total_events=10,
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 1)
        self.assertTrue(jobs[0]['mask'].getRunAndLumis())  # Make sure it has a lumi to process

        # Test the total event limit again
        testSubscription = self.createSubscription(nFiles=5, lumisPerFile=5, twoSites=False)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)

        jobGroups = jobFactory(halt_job_on_file_boundaries=False, splitOnRun=False, events_per_job=60, total_events=179,
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 3)

        # Test the total event limit on the boundary
        testSubscription = self.createSubscription(nFiles=5, lumisPerFile=5, twoSites=False)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)

        jobGroups = jobFactory(halt_job_on_file_boundaries=False, splitOnRun=False, events_per_job=60, total_events=180,
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 3)

        # Test the total event limit just past the boundary
        testSubscription = self.createSubscription(nFiles=5, lumisPerFile=5, twoSites=False)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)

        jobGroups = jobFactory(halt_job_on_file_boundaries=False, splitOnRun=False, events_per_job=60, total_events=181,
                               sperformance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 4)

        return

    def testFileSplitNoHardLimit(self):
        """
        _testFileSplitNoHardLimit_

        Simplest use case, there is only a self limit of events per job which
        the algorithm must adapt to on a file by file basis. At most
        one file per job so we don't have to pass information between files.
        """
        splitter = SplitterFactory()

        # Create 5 files with 7 lumi per file and 100 events per lumi on average.
        testSubscription = self.createSubscription(nFiles=5, lumisPerFile=7, twoSites=False, nEventsPerFile=700)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)

        # First test, the optimal settings are 360 events per job
        # As we have files with 100 events per lumi, this will configure the splitting to
        # 3.6 lumis per job, which rounds to 3, the algorithm always approximates to the lower integer.
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, splitOnRun=True, events_per_job=360,
                               performance=self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 10)

        # Now set the average to 200 events per job
        # This results in the algorithm reducing the lumis per job to 2
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, splitOnRun=True, events_per_job=200,
                               performance=self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 20)

        # Check extremes, process a zero event files with lumis. It must be processed in one job
        testSubscription = self.createSubscription(nFiles=5, lumisPerFile=100, twoSites=False,
                                                   nEventsPerFile=0)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, events_per_job=5000,
                               performance=self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 5)

        # Process files with 10k events per lumi, fallback to one lumi per job. We can't do better
        testSubscription = self.createSubscription(nFiles=5, lumisPerFile=5, twoSites=False,
                                                   nEventsPerFile=50000)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, splitOnRun=True, events_per_job=5000,
                               performance=self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 25)

        # Test total_events limit. (The algorithm cuts off after the lumi that
        # brings the total average event count over -or equal to- total_events.)
        testSubscription = self.createSubscription(nFiles=5, lumisPerFile=3, twoSites=False,
                                                   nEventsPerFile=300)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, splitOnRun=True, events_per_job=250, total_events=750,
                               performance=self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 5)
        self.enforceLimits(filesPerJob=1, jobsPerFile=2, runsPerJob=1)

        return

    def testNoFileSplitNoHardLimit(self):
        """
        _testNoFileSplitNoHardLimit_

        In this case we don't split on file boundaries, check different combination of files
        make sure we make the most of the splitting, e.g. include many zero event files in
        a single job.
        """
        splitter = SplitterFactory()

        # Create 100 files with 7 lumi per file and 0 events per lumi on average.
        testSubscription = self.createSubscription(nFiles=100, lumisPerFile=7, twoSites=False, nEventsPerFile=0)
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)

        # First test, the optimal settings are 360 events per job. As we have files with 0 events per lumi, this will
        # configure the splitting to a single job containing all files
        jobGroups = jobFactory(halt_job_on_file_boundaries=False, splitOnRun=False, events_per_job=360,
                               performance=self.performanceParams)

        # One job in one job group with 100 files
        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 1)
        self.assertEqual(len(jobs[0]['input_files']), 100)

        # Create 7 files, each one with different lumi/event distributions
        testFileset = Fileset(name="FilesetA")
        testFileA = self.createFile("/this/is/file1", 250, 0, 5, "blenheim")
        testFileB = self.createFile("/this/is/file2", 600, 1, 1, "blenheim")
        testFileC = self.createFile("/this/is/file3", 1200, 2, 2, "blenheim")
        testFileD = self.createFile("/this/is/file4", 100, 3, 1, "blenheim")
        testFileE = self.createFile("/this/is/file5", 30, 4, 1, "blenheim")
        testFileF = self.createFile("/this/is/file6", 10, 5, 1, "blenheim")
        testFileG = self.createFile("/this/is/file7", 153, 6, 3, "blenheim")
        testFileset.addFile(testFileA)
        testFileset.addFile(testFileB)
        testFileset.addFile(testFileC)
        testFileset.addFile(testFileD)
        testFileset.addFile(testFileE)
        testFileset.addFile(testFileF)
        testFileset.addFile(testFileG)

        testSubscription = Subscription(fileset=testFileset, workflow=self.testWorkflow,
                                        split_algo="EventAwareLumiByWork", type="Processing")
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)
        # Split the work targeting 150 events per job
        jobGroups = jobFactory(halt_job_on_file_boundaries=False, splitOnRun=False, events_per_job=150,
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 7)

        # Test interactions of this algorithm with splitOnRun = True
        # Make 2 files, one with 3 runs and a second one with the last run of the first
        fileA = File(lfn="/this/is/file1", size=1000, events=2400)
        lumiListA = []
        lumiListB = []
        lumiListC = []
        for lumi in range(8):
            lumiListA.append(1 + lumi)
            lumiListB.append(1 + lumi)
            lumiListC.append(1 + lumi)
        fileA.addRun(Run(1, *lumiListA))
        fileA.addRun(Run(2, *lumiListA))
        fileA.addRun(Run(3, *lumiListA))
        fileA.setLocation("malpaquet")

        fileB = self.createFile('/this/is/file2', 200, 3, 5, "malpaquet")

        testFileset = Fileset(name='FilesetB')
        testFileset.addFile(fileA)
        testFileset.addFile(fileB)
        testSubscription = Subscription(fileset=testFileset, workflow=self.testWorkflow,
                                        split_algo="EventAwareLumiByWork", type="Processing")
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)
        # The settings for this splitting are 700 events per job
        jobGroups = jobFactory(splitOnRun=True, halt_job_on_file_boundaries=False, events_per_job=700,
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 6)
        # Make sure each job has one run
        for job in jobs:
            self.assertEqual(len(job['mask'].getRunAndLumis()), 1)

    def testHardLimitSplitting(self):
        """
        _testHardLimitSplitting_

        Test that we can specify a event limit, the
        algorithm shall take single lumi files with more events than the limit
        and mark them for failure
        """
        splitter = SplitterFactory()

        # Create 3 files, the one in the middle is a "bad" file
        testFileset = Fileset(name="FilesetA")
        testFileA = self.createFile("/this/is/file1", 1000, 0, 5, "blenheim")
        testFileB = self.createFile("/this/is/file2", 1000, 1, 1, "blenheim")
        testFileC = self.createFile("/this/is/file3", 1000, 2, 2, "blenheim")
        testFileset.addFile(testFileA)
        testFileset.addFile(testFileB)
        testFileset.addFile(testFileC)

        testSubscription = Subscription(fileset=testFileset, workflow=self.testWorkflow,
                                        split_algo="EventAwareLumiByWork", type="Processing")
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)

        # Settings are to split on job boundaries, to fail single lumis with more than 800 events
        # and to put 550 events per job
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, splitOnRun=True, events_per_job=550,
                               job_time_limit=9600, performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 5)

        # One job should be failed, the rest should be fine
        for jobNum in (0, 1, 3, 4):
            self.assertFalse(jobs[jobNum].get('failedOnCreation'))
        self.assertTrue(jobs[2]['failedOnCreation'])
        self.assertEqual(jobs[2]['failedReason'], 'File /this/is/file2 has a single lumi 1, in run 1 with too many events 1000 and it would take 12000 sec to run')

        return

    def testHardLimitSplittingOnly(self):
        """
        _testHardLimitSplittingOnly_

        Checks that we can split a set of files where every file has a single
        lumi too big to fit in a runnable job
        """
        splitter = SplitterFactory()

        # Create 3 single-big-lumi files
        testFileset = Fileset(name="FilesetA")
        testFileA = self.createFile("/this/is/file1", 1000, 0, 1, "somese.cern.ch")
        testFileB = self.createFile("/this/is/file2", 1000, 1, 1, "somese.cern.ch")
        testFileC = self.createFile("/this/is/file3", 1000, 2, 1, "somese.cern.ch")
        testFileset.addFile(testFileA)
        testFileset.addFile(testFileB)
        testFileset.addFile(testFileC)

        testSubscription = Subscription(fileset=testFileset, workflow=self.testWorkflow,
                                        split_algo="EventAwareLumiByWork", type="Processing")
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)

        # Fail single lumis with more than 800 events and put 550 events per job
        jobGroups = jobFactory(halt_job_on_file_boundaries=True, splitOnRun=True, events_per_job=550,
                               job_time_limit=9600, performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 3)
        for job in jobs:
            self.assertTrue(job['failedOnCreation'])
            self.assertIn(' with too many events 1000 and it would take 12000 sec to run', job['failedReason'])

        return

    def testLumiMask(self):
        """
        _testLumiMask_

        Test that we can use a lumi-mask to filter good runs/lumis.
        """
        splitter = SplitterFactory()

        # Create 3 files with 100 events per lumi:
        # - file1 with 1 run  of 8 lumis
        # - file2 with 2 runs of 2 lumis each
        # - file3 with 1 run  of 5 lumis
        fileA = File(lfn="/this/is/file1", size=1000, events=800)
        fileB = File(lfn="/this/is/file2", size=1000, events=400)
        fileC = File(lfn="/this/is/file3", size=1000, events=500)

        lumiListA = []
        for lumi in range(8):
            lumiListA.append(10 + lumi)
        fileA.addRun(Run(1, *lumiListA))
        fileA.setLocation("somese.cern.ch")
        lumiListB1 = []
        lumiListB2 = []
        for lumi in range(2):
            lumiListB1.append(20 + lumi)
            lumiListB2.append(30 + lumi)
        fileB.addRun(Run(2, *lumiListB1))
        fileB.addRun(Run(3, *lumiListB2))
        fileB.setLocation("somese.cern.ch")
        lumiListC = []
        for lumi in range(5):
            lumiListC.append(40 + lumi)
        fileC.addRun(Run(4, *lumiListC))
        fileC.setLocation("somese.cern.ch")

        testFileset = Fileset(name='Fileset')
        testFileset.addFile(fileA)
        testFileset.addFile(fileB)
        testFileset.addFile(fileC)

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=self.testWorkflow,
                                        split_algo="EventAwareLumiByWork",
                                        type="Processing")
        jobFactory = splitter(package="WMCore.DataStructs",
                              subscription=testSubscription)

        # Use a lumi-mask = {1: [[10,14]], 2: [[20,21]], 4: [[40,41]]}
        jobGroups = jobFactory(halt_job_on_file_boundaries=False,
                               splitOnRun=False,
                               events_per_job=850,
                               runs=['1', '2', '4'],
                               lumis=['10,14', '20,21', '40,41'],
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1, "There should be only one job group")
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 2, "Two jobs must be in the jobgroup")
        processedLumis = LumiList()
        for job in jobs:
            processedLumis += LumiList(compactList=job['mask'].getRunAndLumis())
        correctLumis = LumiList(compactList={1: [[10, 14]], 2: [[20, 21]], 4: [[40, 41]]})
        self.assertEqual(processedLumis.getCMSSWString(), correctLumis.getCMSSWString())

    def testRunWhiteList(self):
        """
        _testRunWhiteList_

        Test that we can use a run white list to filter good runs/lumis.
        """
        splitter = SplitterFactory()

        # Create 3 files with 100 events per lumi:
        # - file1 with 1 run  of 8 lumis
        # - file2 with 2 runs of 2 lumis each
        # - file3 with 1 run  of 5 lumis
        fileA = File(lfn="/this/is/file1", size=1000, events=800)
        fileB = File(lfn="/this/is/file2", size=1000, events=400)
        fileC = File(lfn="/this/is/file3", size=1000, events=500)

        lumiListA = []
        for lumi in range(8):
            lumiListA.append(10 + lumi)
        fileA.addRun(Run(1, *lumiListA))
        fileA.setLocation("somese.cern.ch")
        lumiListB1 = []
        lumiListB2 = []
        for lumi in range(2):
            lumiListB1.append(20 + lumi)
            lumiListB2.append(30 + lumi)
        fileB.addRun(Run(2, *lumiListB1))
        fileB.addRun(Run(3, *lumiListB2))
        fileB.setLocation("somese.cern.ch")
        lumiListC = []
        for lumi in range(5):
            lumiListC.append(40 + lumi)
        fileC.addRun(Run(4, *lumiListC))
        fileC.setLocation("somese.cern.ch")

        testFileset = Fileset(name='Fileset')
        testFileset.addFile(fileA)
        testFileset.addFile(fileB)
        testFileset.addFile(fileC)

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=self.testWorkflow,
                                        split_algo="EventAwareLumiByWork",
                                        type="Processing")
        jobFactory = splitter(package="WMCore.DataStructs",
                              subscription=testSubscription)

        # Split with no breaks
        jobGroups = jobFactory(halt_job_on_file_boundaries=False,
                               splitOnRun=False,
                               events_per_job=725,
                               runWhitelist=[1, 4],
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 2)
        for job in jobs:
            for run in job['mask'].getRunAndLumis():
                self.assertIn(run, [1, 4])

        # Re-split with a break on runs
        jobGroups = jobFactory(halt_job_on_file_boundaries=False,
                               splitOnRun=True,
                               events_per_job=595,
                               runWhitelist=[1, 3, 4],
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 4)
        self.enforceLimits(jobs=jobs, runsPerJob=1)
        for job in jobs:
            for run in job['mask'].getRunAndLumis():
                self.assertIn(run, [1, 3, 4])

        # Re-split with a break on files
        jobGroups = jobFactory(halt_job_on_file_boundaries=True,
                               splitOnRun=False,
                               events_per_job=595,
                               runWhitelist=[1, 2, 3],
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 3)
        self.enforceLimits(jobs=jobs, filesPerJob=1)
        for job in jobs:
            for run in job['mask'].getRunAndLumis():
                self.assertIn(run, [1, 2, 3])

    def testLumiMaskAndWhitelist(self):
        """
        _testLumiMaskAndWhitelist_

        Test that we can use a lumi-mask to filter good runs/lumis.
        """
        splitter = SplitterFactory()

        # Create 3 files with 100 events per lumi:
        # - file1 with 1 run  of 8 lumis
        # - file2 with 2 runs of 2 lumis each
        # - file3 with 1 run  of 5 lumis
        fileA = File(lfn="/this/is/file1", size=1000, events=800)
        fileB = File(lfn="/this/is/file2", size=1000, events=400)
        fileC = File(lfn="/this/is/file3", size=1000, events=500)

        lumiListA = []
        for lumi in range(8):
            lumiListA.append(10 + lumi)
        fileA.addRun(Run(1, *lumiListA))
        fileA.setLocation("somese.cern.ch")
        lumiListB1 = []
        lumiListB2 = []
        for lumi in range(2):
            lumiListB1.append(20 + lumi)
            lumiListB2.append(30 + lumi)
        fileB.addRun(Run(2, *lumiListB1))
        fileB.addRun(Run(3, *lumiListB2))
        fileB.setLocation("somese.cern.ch")
        lumiListC = []
        for lumi in range(5):
            lumiListC.append(40 + lumi)
        fileC.addRun(Run(4, *lumiListC))
        fileC.setLocation("somese.cern.ch")

        testFileset = Fileset(name='Fileset')
        testFileset.addFile(fileA)
        testFileset.addFile(fileB)
        testFileset.addFile(fileC)

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=self.testWorkflow,
                                        split_algo="EventAwareLumiByWork",
                                        type="Processing")
        jobFactory = splitter(package="WMCore.DataStructs",
                              subscription=testSubscription)

        # Use a lumi-mask = {1: [[10,14]], 2: [[20,21]], 4: [[40,41]]}
        jobGroups = jobFactory(halt_job_on_file_boundaries=False,
                               splitOnRun=False,
                               events_per_job=850,
                               runs=['1', '2', '4'],
                               lumis=['10,14', '20,21', '40,41'],
                               runWhitelist=[1, 4],
                               performance=self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 1)
        self.assertEqual(jobs[0]['mask'].getRunAndLumis(), {1: [[10, 14]], 4: [[40, 41]]})

    def testLumiCorrections(self):
        """
        _testLumiCorrections_

        Test the splitting algorithm can handle lumis which cross multiple files.
        No need for applyLumiCorrection=True
        """

        splitter = SplitterFactory()
        testSubscription = self.createSubscription(nFiles=2, lumisPerFile=2, twoSites=False, nEventsPerFile=150)
        files = testSubscription.getFileset().getFiles()
        self.assertEqual(len(files), 2)

        # Two files with 2 lumis each: file0 has run0 and lumis 0,1 - file1 has run1 and lumis 2,3 - each 150 events
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)

        jobGroups = jobFactory(events_per_job=50, halt_job_on_file_boundaries=False, splitOnRun=False,
                               performance=self.performanceParams)

        # The splitting algorithm will assume 75 events per lumi so we will have one job per lumi
        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 4)

        # Recreate the same subscription as before
        testSubscription = self.createSubscription(nFiles=2, lumisPerFile=2, twoSites=False, nEventsPerFile=150)
        files = testSubscription.getFileset().getFiles()
        # Now modifyng and adding duplicated lumis.
        for runObj in files[0]['runs']:
            if runObj.run == 0:
                # continue
                runObj.appendLumi(42)
        for runObj in files[1]['runs']:
            if runObj.run == 1:
                # continue
                runObj.run = 0
                runObj.appendLumi(42)
        files[1]['locations'] = {'blenheim'}
        jobFactory = splitter(package="WMCore.DataStructs", subscription=testSubscription)
        jobGroups = jobFactory(events_per_job=50, halt_job_on_file_boundaries=True, performance=self.performanceParams)

        # Now we will have: file0: Run0 and lumis [0, 1, 42] file1: Run0 and lumis [2, 3, 42]
        # With 50 events per lumi, one job per lumi, one will have two files on lumi 42
        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 5)

        n1files = 0
        n2files = 0
        lumi1files = []
        lumi2files = []

        for job in jobs:
            runLumis = job['mask'].getRunAndLumis()
            lumis = runLumis[0]
            self.assertEqual(len(runLumis), 1)
            self.assertEqual(len(lumis), 1)
            self.assertEqual(lumis[0][0], lumis[0][1])  # Make sure only one lumi per job
            if len(job['input_files']) == 1:
                n1files += 1
                lumi1files.append(lumis[0][0])
            elif len(job['input_files']) == 2:
                n2files += 1
                lumi2files.append(lumis[0][0])
            else:
                self.fail("At least one job has nFiles =! 1, 2")

        self.assertEqual(n1files, 4)
        self.assertEqual(n2files, 1)
        self.assertItemsEqual(lumi1files, [0, 1, 2, 3])
        self.assertItemsEqual(lumi2files, [42])

    def enforceLimits(self, jobs=None, filesPerJob=None, jobsPerFile=None, runsPerJob=None):
        """
        Args:
            jobs: the list of jobs to check
            filesPerJob: The maximum number of files that can be in one job
            jobsPerFile: The maximum number of jobs that a particular file can appear in
            runsPerJob: The maximum number of runs that can be in a job

        Returns: nothing

        Check the various limits on job and file distribution, raise assertion errors if they are violated
        """

        jobs = jobs or []
        jobsPerFileMap = Counter()
        for job in jobs:
            if filesPerJob:
                self.assertLessEqual(len(job['input_files']), filesPerJob)
            if runsPerJob:
                self.assertLessEqual(len(job['mask'].getRunAndLumis()), runsPerJob)
            for f in job['input_files']:
                jobsPerFileMap[f['lfn']] += 1
        for lfn in jobsPerFileMap:
            if jobsPerFile:
                self.assertLessEqual(jobsPerFileMap[lfn], jobsPerFile)

        return


if __name__ == '__main__':
    unittest.main()



"""
Unit tests for the WMCore/MicroService/DataStructs/MSRuleCleanerWflow.py module
"""
from __future__ import division, print_function

import json
import os
import unittest

from WMCore.MicroService.MSRuleCleaner.MSRuleCleanerWflow import MSRuleCleanerWflow


def getTestFile(partialPath):
    """
    Returns the absolute path for the test json file
    """
    normPath = os.path.normpath(os.path.join(os.path.dirname(__file__), '..', '..', '..', '..'))
    return os.path.join(normPath, partialPath)


class MSRuleCleanerWflowTest(unittest.TestCase):
    """
    Test the very basic functionality of the MSRuleCleanerWflow module
    """

    def setUp(self):
        self.maxDiff = None
        self.taskChainFile = getTestFile('data/ReqMgr/requests/Static/TaskChainRequestDump.json')
        self.stepChainFile = getTestFile('data/ReqMgr/requests/Static/StepChainRequestDump.json')
        self.reRecoFile = getTestFile('data/ReqMgr/requests/Static/ReRecoRequestDump.json')
        self.includeParentsFile = getTestFile('data/ReqMgr/requests/Static/IncludeParentsRequestDump.json')
        self.multiPUFile = getTestFile('data/ReqMgr/requests/Static/MultiPURequestDump.json')
        self.reqRecordsFile = getTestFile('data/ReqMgr/requests/Static/BatchRequestsDump.json')
        with open(self.reqRecordsFile, encoding="utf8") as fd:
            self.reqRecords = json.load(fd)
        with open(self.taskChainFile, encoding="utf8") as fd:
            self.taskChainReq = json.load(fd)
        with open(self.stepChainFile, encoding="utf8") as fd:
            self.stepChainReq = json.load(fd)
        with open(self.reRecoFile, encoding="utf8") as fd:
            self.reRecoReq = json.load(fd)
        with open(self.includeParentsFile, encoding="utf8") as fd:
            self.includeParentsReq = json.load(fd)
        with open(self.multiPUFile, encoding="utf8") as fd:
            self.multiPUReq = json.load(fd)
        super(MSRuleCleanerWflowTest, self).setUp()

    def testTaskChainDefaults(self):
        """
        Test creating a MSRuleCleanerWflow object out of a TaskChain request dictionary
        """
        wflow = MSRuleCleanerWflow({})
        self.assertEqual(wflow["RequestName"], None)
        self.assertEqual(wflow["RequestType"], None)
        self.assertEqual(wflow["SubRequestType"], None)
        self.assertEqual(wflow["RequestStatus"], None)
        self.assertEqual(wflow["OutputDatasets"], [])
        self.assertEqual(wflow["RulesToClean"], {})
        self.assertEqual(wflow["CleanupStatus"], {})
        self.assertEqual(wflow["TransferDone"], False)
        self.assertEqual(wflow["TargetStatus"], None)
        self.assertEqual(wflow["ParentageResolved"], True)
        self.assertEqual(wflow["PlineMarkers"], None)
        self.assertEqual(wflow["IsClean"], False)
        self.assertEqual(wflow["IsLogDBClean"], False)
        self.assertEqual(wflow["IsArchivalDelayExpired"], False)
        self.assertEqual(wflow["ForceArchive"], False)
        self.assertEqual(wflow["RequestTransition"], [])
        self.assertEqual(wflow['IncludeParents'], False)
        self.assertEqual(wflow['InputDataset'], None)
        self.assertEqual(wflow['ParentDataset'], [])
        self.assertEqual(wflow['TapeRulesStatus'], [])
        self.assertEqual(wflow['StatusAdvanceExpiredMsg'], "")

    def testTaskChain(self):
        # Test Taskchain:
        wflow = MSRuleCleanerWflow(self.taskChainReq)
        expectedWflow = {'CleanupStatus': {},
                         'ForceArchive': False,
                         'IncludeParents': False,
                         'InputDataset': '/JetHT/Run2012C-v1/RAW',
                         'IsArchivalDelayExpired': False,
                         'IsClean': False,
                         'IsLogDBClean': False,
                         'OutputDatasets': [
                             '/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/RECO',
                             '/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/DQMIO',
                             '/JetHT/CMSSW_7_2_0-SiStripCalMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             '/JetHT/CMSSW_7_2_0-SiStripCalZeroBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             '/JetHT/CMSSW_7_2_0-TkAlMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO'],
                         'ParentDataset': [],
                         'ParentageResolved': True,
                         'PlineMarkers': None,
                         'RequestName': 'TaskChain_LumiMask_multiRun_HG2011_Val_201029_112735_5891',
                         'RequestStatus': 'announced',
                         'RequestTransition': [{'DN': '',
                                                'Status': 'new',
                                                'UpdateTime': 1606723304},
                                               {'DN': '', 'Status': 'assignment-approved',
                                                'UpdateTime': 1606723305},
                                               {'DN': '', 'Status': 'assigned', 'UpdateTime': 1606723306},
                                               {'DN': '', 'Status': 'staging', 'UpdateTime': 1606723461},
                                               {'DN': '', 'Status': 'staged', 'UpdateTime': 1606723590},
                                               {'DN': '', 'Status': 'acquired', 'UpdateTime': 1606723968},
                                               {'DN': '', 'Status': 'running-open', 'UpdateTime': 1606724572},
                                               {'DN': '', 'Status': 'running-closed', 'UpdateTime': 1606724573},
                                               {'DN': '', 'Status': 'completed', 'UpdateTime': 1607018413},
                                               {'DN': '', 'Status': 'closed-out', 'UpdateTime': 1607347706},
                                               {'DN': '', 'Status': 'announced', 'UpdateTime': 1607359514}],
                         'RequestType': 'TaskChain',
                         'SubRequestType': '',
                         'RulesToClean': {},
                         'TargetStatus': None,
                         'TransferDone': False,
                         'TransferTape': False,
                         'TapeRulesStatus': [],
                         'StatusAdvanceExpiredMsg': ""}
        self.assertDictEqual(wflow, expectedWflow)

    def testReReco(self):
        # Test ReReco workflow:
        wflow = MSRuleCleanerWflow(self.reRecoReq)
        expectedWflow = {'CleanupStatus': {},
                         'ForceArchive': False,
                         'IncludeParents': False,
                         'InputDataset': '/SingleElectron/Run2017F-v1/RAW',
                         'IsArchivalDelayExpired': False,
                         'IsClean': False,
                         'IsLogDBClean': False,
                         'OutputDatasets': ['/SingleElectron/Run2017F-09Aug2019_UL2017_EcalRecovery-v1/MINIAOD',
                                            '/SingleElectron/Run2017F-EcalUncalWElectron-09Aug2019_UL2017_EcalRecovery-v1/ALCARECO',
                                            '/SingleElectron/Run2017F-EcalUncalZElectron-09Aug2019_UL2017_EcalRecovery-v1/ALCARECO',
                                            '/SingleElectron/Run2017F-09Aug2019_UL2017_EcalRecovery-v1/AOD',
                                            '/SingleElectron/Run2017F-EcalESAlign-09Aug2019_UL2017_EcalRecovery-v1/ALCARECO',
                                            '/SingleElectron/Run2017F-09Aug2019_UL2017_EcalRecovery-v1/DQMIO',
                                            '/SingleElectron/Run2017F-HcalCalIterativePhiSym-09Aug2019_UL2017_EcalRecovery-v1/ALCARECO'],
                         'ParentDataset': [],
                         'ParentageResolved': True,
                         'PlineMarkers': None,
                         'RequestName': 'pdmvserv_Run2017F-v1_SingleElectron_09Aug2019_UL2017_EcalRecovery_200506_120455_3146',
                         'RequestStatus': 'completed',
                         'RequestTransition': [{'DN': '',
                                                'Status': 'new',
                                                'UpdateTime': 1588759495},
                                               {'DN': '', 'Status': 'assignment-approved',
                                                'UpdateTime': 1588759500},
                                               {'DN': '', 'Status': 'assigned', 'UpdateTime': 1588760963},
                                               {'DN': '', 'Status': 'staging', 'UpdateTime': 1588761258},
                                               {'DN': '', 'Status': 'staged', 'UpdateTime': 1588761688},
                                               {'DN': '', 'Status': 'acquired', 'UpdateTime': 1588762166},
                                               {'DN': '', 'Status': 'running-open', 'UpdateTime': 1588763098},
                                               {'DN': '', 'Status': 'running-closed', 'UpdateTime': 1588775354},
                                               {'DN': '', 'Status': 'completed', 'UpdateTime': 1589041121}],
                         'RequestType': 'ReReco',
                         'SubRequestType': '',
                         'RulesToClean': {},
                         'TargetStatus': None,
                         'TransferDone': False,
                         'TransferTape': False,
                         'TapeRulesStatus': [],
                         'StatusAdvanceExpiredMsg': ""}
        self.assertDictEqual(wflow, expectedWflow)

    def testIncludeParents(self):
        # Test include parents::
        wflow = MSRuleCleanerWflow(self.includeParentsReq)
        expectedWflow = {'CleanupStatus': {},
                         'ForceArchive': False,
                         'IncludeParents': True,
                         'InputDataset': '/Cosmics/Commissioning2015-PromptReco-v1/RECO',
                         'IsArchivalDelayExpired': False,
                         'IsClean': False,
                         'IsLogDBClean': False,
                         'OutputDatasets': [
                             '/Cosmics/Integ_Test-CosmicSP-StepChain_InclParents_HG2004_Val_Privv12-v11/RAW-RECO'],
                         'ParentDataset': [],
                         'ParentageResolved': False,
                         'PlineMarkers': None,
                         'RequestName': 'amaltaro_StepChain_InclParents_April2020_Val_200414_120713_81',
                         'RequestStatus': 'completed',
                         'RequestTransition': [{'DN': '',
                                                'Status': 'new',
                                                'UpdateTime': 1586858833},
                                               {'DN': '', 'Status': 'assignment-approved',
                                                'UpdateTime': 1586858834},
                                               {'DN': '', 'Status': 'assigned', 'UpdateTime': 1586858835},
                                               {'DN': '', 'Status': 'staging', 'UpdateTime': 1586859358},
                                               {'DN': '', 'Status': 'staged', 'UpdateTime': 1586859733},
                                               {'DN': '', 'Status': 'acquired', 'UpdateTime': 1586860322},
                                               {'DN': '', 'Status': 'running-open', 'UpdateTime': 1586860927},
                                               {'DN': '', 'Status': 'running-closed', 'UpdateTime': 1586861535},
                                               {'DN': '', 'Status': 'completed', 'UpdateTime': 1586863942}],
                         'RequestType': 'StepChain',
                         'SubRequestType': '',
                         'RulesToClean': {},
                         'TargetStatus': None,
                         'TransferDone': False,
                         'TransferTape': False,
                         'TapeRulesStatus': [],
                         'StatusAdvanceExpiredMsg': ""}
        self.assertDictEqual(wflow, expectedWflow)

if __name__ == '__main__':
    unittest.main()



"""
_MarkBlocksDeleted_

Oracle implementation of PhEDExInjector.MarkBlocksDeleted

"""

from __future__ import division
from __future__ import print_function

from WMComponent.RucioInjector.Database.MySQL.MarkBlocksDeleted import MarkBlocksDeleted as MySQLBase


class MarkBlocksDeleted(MySQLBase):
    pass



#! /bin/env python
"""
Test case for McM service
"""

import unittest
from nose.plugins.attrib import attr
from WMCore.Services.McM.McM import McM

prepID = 'BTV-Upg2023SHCAL14DR-00002'


class McMTest(unittest.TestCase):
    """
    Unit tests for McM Service
    """

    @attr("integration")
    def testHistory(self):
        """
        Test that the history URL is working
        """

        history = None
        with McM() as mcm:
            history = mcm.getHistory(prepID=prepID)

        isAnnounced = False
        for entry in history:
            if entry['action'] == 'set status' and entry['step'] == 'announced':
                isAnnounced = True

        self.assertTrue(isAnnounced)

    @attr("integration")
    def testRequest(self):
        """
        Test that the request URL is working
        """
        request = None
        with McM() as mcm:
            request = mcm.getRequest(prepID=prepID)

        self.assertTrue('total_events' in request)

    @attr("integration")
    def testHistoryDevelopmentEnv(self):
        """
        Test that the history URL is working.
        McM Development has been migrated to the new SSO
        (Keycloak)
        """

        history = None
        development_url: str = "https://cms-pdmv-dev.cern.ch/mcm"
        with McM(url=development_url) as mcm:
            history = mcm.getHistory(prepID=prepID)

        isAnnounced = False
        for entry in history:
            if entry['action'] == 'set status' and entry['step'] == 'announced':
                isAnnounced = True

        self.assertTrue(isAnnounced)

    @attr("integration")
    def testRequestDevelopmentEnv(self):
        """
        Test that the request URL is working.
        McM Development has been migrated to the new SSO
        (Keycloak)
        """
        request = None
        development_url: str = "https://cms-pdmv-dev.cern.ch/mcm"
        with McM(url=development_url) as mcm:
            request = mcm.getRequest(prepID=prepID)

        self.assertTrue('total_events' in request)


if __name__ == '__main__':
    unittest.main()



"""
Unit tests for the WMCore/MicroService/DataStructs/DQMHarvestWorkflow module
"""
from __future__ import division, print_function

import unittest
from copy import deepcopy

from WMCore.MicroService.MSTransferor.DataStructs.DQMHarvestWorkflow import DQMHarvestWorkflow


class DQMHarvestWorkflowTest(unittest.TestCase):
    """
    Test the very basic functionality of the DQMHarvestWorkflow module
    """

    def setUp(self):
        """
        Defined some basic data structs to use in the unit tests
        """
        self.primaryDict = {"block_A": {"blockSize": 1, "locations": ["Site_A", "Site_B"]},
                            "block_B": {"blockSize": 2, "locations": ["Site_B", "Site_C"]}}
        self.dqmSpec = {"RequestType": "ReReco",
                        "InputDataset": "/rereco/input-dataset/tier",
                        "Campaign": "any-campaign",
                        "RequestName": "whatever_name",
                        "DbsUrl": "a_dbs_url",
                        "SiteWhitelist": ["Site_A", "Site_B", "Site_C"],
                        "SiteBlacklist": ["Site_B"]}

    def tearDown(self):
        pass

    def testInstance(self):
        """
        Test object instance type
        """
        wflow = DQMHarvestWorkflow(self.dqmSpec['RequestName'], deepcopy(self.dqmSpec))
        self.assertIsInstance(wflow, DQMHarvestWorkflow)

    def testDQMHarvestWflow(self):
        """
        Test loading a DQMHarvest like request into Workflow object
        """
        wflow = DQMHarvestWorkflow(self.dqmSpec['RequestName'], deepcopy(self.dqmSpec))
        self.assertEqual(wflow.getName(), self.dqmSpec['RequestName'])
        self.assertEqual(wflow.getDbsUrl(), self.dqmSpec['DbsUrl'])
        self.assertCountEqual(wflow.getSitelist(), ["Site_A", "Site_C"])
        self.assertCountEqual(wflow.getCampaigns(), [self.dqmSpec["Campaign"]])
        self.assertEqual(wflow.getInputDataset(), self.dqmSpec["InputDataset"])
        self.assertCountEqual(wflow.getPileupDatasets(), set())
        self.assertFalse(wflow.hasParents())
        self.assertEqual(wflow.getParentDataset(), "")
        self.assertEqual(wflow.getPrimaryBlocks(), {})
        self.assertEqual(wflow.getSecondarySummary(), {})
        self.assertEqual(wflow.getParentBlocks(), {})
        self.assertEqual(wflow._getValue("NoKey"), None)
        self.assertEqual(len(wflow.getDataCampaignMap()), 1)

    def testGetInputData(self):
        """
        Test the `getInputData` method for this template, which
        is supposed to return a list of input blocks
        """
        wflow = DQMHarvestWorkflow(self.dqmSpec['RequestName'], deepcopy(self.dqmSpec))
        wflow.setPrimaryBlocks(self.primaryDict)
        inputBlocks, blockSize = wflow.getInputData()
        # this MSTransferor policy deals with the whole container
        self.assertEqual(inputBlocks, [self.dqmSpec["InputDataset"]])
        self.assertEqual(blockSize, 3)

    def testGetRucioGrouping(self):
        """
        Test the `getRucioGrouping` method, which is supposed to return
        a basic string with the Rucio grouping for this template (static
        output).
        """
        wflow = DQMHarvestWorkflow(self.dqmSpec['RequestName'], deepcopy(self.dqmSpec))
        self.assertEqual(wflow.getRucioGrouping(), "ALL")


if __name__ == '__main__':
    unittest.main()



"""
Web tools.
Author: Valentin Kuznetsov <vkuznet {AT} gmail [DOT] com>
"""

from __future__ import print_function
from builtins import object, int

# system modules
import json
import logging
import os
import sys
import cgi
import types
from datetime import datetime, timedelta
from json import JSONEncoder
from time import mktime
from wsgiref.handlers import format_date_time

# cherrypy modules
import cherrypy
from cherrypy import log as cplog
from cherrypy import expose

try:
    from Cheetah.Template import Template
except:
    pass
try:
    import jinja2
except:
    pass


def quote(data):
    """
    Sanitize the data using cgi.escape.
    """
    if isinstance(data, (int, float, dict, list)):
        res = data
    else:
        try:
            if data:
                res = cgi.escape(data, quote=True)
            else:
                res = ""
        except Exception as exc:
            print("Unable to cgi.escape(%s, quote=True)" % data)
            print(exc)
            res = ""
    return res

class Page(object):
    """
    __Page__

    Page is a base class that holds a configuration
    """
    def __init__(self):
        self.name = "Page"

    def warning(self, msg):
        """Define warning log"""
        if  msg:
            self.log(msg, logging.WARNING)

    def exception(self, msg):
        """Define exception log"""
        if  msg:
            self.log(msg, logging.ERROR)

    def debug(self, msg):
        """Define debug log"""
        if  msg:
            self.log(msg, logging.DEBUG)

    def info(self, msg):
        """Define info log"""
        if  msg:
            self.log(msg, logging.INFO)

    def log(self, msg, severity):
        """Define log level"""
        if not isinstance(msg, str):
            msg = str(msg)
        if  msg:
            cplog(msg, context=self.name,
                    severity=severity, traceback=False)

class TemplatedPage(Page):
    """
    TemplatedPage is a class that provides simple Cheetah templating
    """
    def __init__(self, config):
        Page.__init__(self)
        tmpldir  = os.environ.get('RM_TMPLPATH', os.getcwd()+'/templates')
        self.templatedir = config.get('tmpldir', tmpldir)
        self.name = "TemplatedPage"
        self.base = config.get('base', '')
        self.jinja = True if 'jinja2' in sys.modules else False
        if  self.jinja:
            templates = 'JINJA'
        else:
            templates = 'Cheetah'
        self.log("### ReqMgr uses %s templates" % templates, logging.INFO)
        self.log("Templates are located in: %s" % self.templatedir, logging.INFO)

    def templatepage(self, ifile=None, *args, **kwargs):
        """Choose template page handler based on templates engine"""
        if  self.jinja and self.templatedir.find("jinja") != -1:
            return self.templatepage_jinja(ifile, *args, **kwargs)
        return self.templatepage_cheetah(ifile, *args, **kwargs)

    def templatepage_cheetah(self, ifile=None, *args, **kwargs):
        """
        Template page method.
        """
        search_list = [{'base':self.base}]
        if len(args) > 0:
            search_list.append(args)
        if len(kwargs) > 0:
            search_list.append(kwargs)
        templatefile = os.path.join(self.templatedir, ifile + '.tmpl')
        if os.path.exists(templatefile):
            # little workaround to fix '#include'
            search_list.append({'templatedir': self.templatedir})
            template = Template(file=templatefile, searchList=search_list)
            return template.respond()

        else:
            self.warning("%s not found at %s" % (ifile, self.templatedir))
            return "Template %s not known" % ifile

    def templatepage_jinja(self, ifile=None, *args, **kwargs):
        """
        Template page method.
        """
        env = jinja2.Environment(loader=jinja2.FileSystemLoader(self.templatedir))
        for arg in args:
            kwargs.update(**arg)
        kwargs.update(**{"quote":quote})
        tmpl = os.path.join(self.templatedir, ifile + '.tmpl')
        if  os.path.exists(tmpl):
            template = env.get_template(ifile + '.tmpl')
            return template.render(kwargs)
        else:
            self.warning("%s not found at %s" % (ifile, self.templatedir))
            return "Template %s not known" % ifile

def exposetext (func):
    """CherryPy expose Text decorator"""
    @expose
    def wrapper (self, *args, **kwds):
        """Decorator wrapper"""
        data = func (self, *args, **kwds)
        cherrypy.response.headers['Content-Type'] = "text/plain"
        return data
    return wrapper

def jsonstreamer(func):
    """JSON streamer decorator"""
    def wrapper (self, *args, **kwds):
        """Decorator wrapper"""
        cherrypy.response.headers['Content-Type'] = "application/json"
        func._cp_config = {'response.stream': True}
        head, data = func (self, *args, **kwds)
        yield json.dumps(head)[:-1] # do not yield }
        yield ', "data": ['
        if  isinstance(data, dict):
            for chunk in JSONEncoder().iterencode(data):
                yield chunk
        elif  isinstance(data, list) or isinstance(data, types.GeneratorType):
            sep = ''
            for rec in data:
                if  sep:
                    yield sep
                for chunk in JSONEncoder().iterencode(rec):
                    yield chunk
                if  not sep:
                    sep = ', '
        else:
            msg = 'jsonstreamer, improper data type %s' % type(data)
            raise Exception(msg)
        yield ']}'
    return wrapper

def exposejson (func):
    """CherryPy expose JSON decorator"""
    @expose
    def wrapper (self, *args, **kwds):
        """Decorator wrapper"""
        encoder = JSONEncoder()
        data = func (self, *args, **kwds)
        cherrypy.response.headers['Content-Type'] = "text/json"
        try:
            jsondata = encoder.encode(data)
            return jsondata
        except Exception as exp:
            Exception("Fail to JSONtify obj '%s' type '%s', %s" \
                % (data, type(data), exp))
    return wrapper

def exposejs (func):
    """CherryPy expose JavaScript decorator"""
    @expose
    def wrapper (self, *args, **kwds):
        """Decorator wrapper"""
        data = func (self, *args, **kwds)
        cherrypy.response.headers['Content-Type'] = "text/javascript"
        return data
    return wrapper

def exposecss (func):
    """CherryPy expose CSS decorator"""
    @expose
    def wrapper (self, *args, **kwds):
        """Decorator wrapper"""
        data = func (self, *args, **kwds)
        cherrypy.response.headers['Content-Type'] = "text/css"
        return data
    return wrapper

def make_timestamp(seconds=0):
    """Create timestamp"""
    then = datetime.now() + timedelta(seconds=seconds)
    return mktime(then.timetuple())

def make_rfc_timestamp(seconds=0):
    """Create RFC timestamp"""
    return format_date_time(make_timestamp(seconds))



#!/usr/bin/env python
from __future__ import (division, print_function)
import unittest

from WMCore.Services.FWJRDB.FWJRDBAPI import FWJRDBAPI
from WMQuality.TestInitCouchApp import TestInitCouchApp


SAMPLE_FWJR = {'fallbackFiles': [],
 'skippedFiles': ['/store/data/Run2011A/Cosmics/RAW/v1/000/160/960/E8099605-8853-E011-A848-0030487A18F3.root',
                  '/store/data/Run2011A/Cosmics/RAW/v1/000/160/960/E8099605-8853-E011-A848-0030487A18F4.root'],
 'steps': {'cmsRun1': {'analysis': {},
                       'cleanup': {},
                       'errors': [],
                       'input': {'source': [{'catalog': '',
                                             'events': 6893,
                                             'guid': 'E8099605-8853-E011-A848-0030487A18F2',
                                             'input_source_class': 'PoolSource',
                                             'input_type': 'primaryFiles',
                                             'lfn': '/store/data/Run2011A/Cosmics/RAW/v1/000/160/960/E8099605-8853-E011-A848-0030487A18F2.root',
                                             'module_label': 'source',
                                             'pfn': 'root://eoscms.cern.ch//eos/cms/store/data/Run2011A/Cosmics/RAW/v1/000/160/960/E8099605-8853-E011-A848-0030487A18F2.root',
                                             'runs': {'160960': [164,
                                                                 165]}},
                                            ]},
                       'logs': {},
                       'output': {'ALCARECOStreamDtCalib': [{'InputPFN': '/pool/condor/dir_2661042/glide_qqd8kp/execute/dir_5444/job/WMTaskSpace/cmsRun1/ALCARECOStreamDtCalib.root',
                                                             'OutputPFN': 'root://eoscms.cern.ch//eos/cms/store/unmerged/CMSSW_7_0_0_pre11/Cosmics/ALCARECO/DtCalib-RECOCOSD_TaskChain_Data_pile_up_test-v1/00000/ECCFE421-08CB-E511-9F4C-02163E017804.root',
                                                             'StageOutCommand': 'rfcp-CERN',
                                                             'acquisitionEra': 'CMSSW_7_0_0_pre11',
                                                             'async_dest': None,
                                                             'branch_hash': 'c1e135af4ac2eb2b803bb6487be2c80f',
                                                             'catalog': '',
                                                             'checksums': {'adler32': 'e503b8b9',
                                                                           'cksum': '2641269665'},
                                                             'configURL': 'https://cmsweb.cern.ch/couchdb;;reqmgr_config_cache;;5f4811e9ccd63d563cd62572350f0db8',
                                                             'dataset': {'applicationName': 'cmsRun',
                                                                         'applicationVersion': 'CMSSW_7_0_0_pre11',
                                                                         'dataTier': 'ALCARECO',
                                                                         'primaryDataset': 'Cosmics',
                                                                         'processedDataset': 'CMSSW_7_0_0_pre11-DtCalib-RECOCOSD_TaskChain_Data_pile_up_test-v1'},
                                                             'events': 0,
                                                             'globalTag': 'GR_R_62_V3::All',
                                                             'guid': 'ECCFE421-08CB-E511-9F4C-02163E017804',
                                                             'input': ['/store/data/Run2011A/Cosmics/RAW/v1/000/160/960/E8099605-8853-E011-A848-0030487A18F2.root'],
                                                             'inputPath': '/Cosmics/Run2011A-v1/RAW',
                                                             'inputpfns': ['root://eoscms.cern.ch//eos/cms/store/data/Run2011A/Cosmics/RAW/v1/000/160/960/E8099605-8853-E011-A848-0030487A18F2.root'],
                                                             'lfn': '/store/unmerged/CMSSW_7_0_0_pre11/Cosmics/ALCARECO/DtCalib-RECOCOSD_TaskChain_Data_pile_up_test-v1/00000/ECCFE421-08CB-E511-9F4C-02163E017804.root',
                                                             'location': None,
                                                             'merged': False,
                                                             'module_label': 'ALCARECOStreamDtCalib',
                                                             'ouput_module_class': 'PoolOutputModule',
                                                             'pfn': '/pool/condor/dir_2661042/glide_qqd8kp/execute/dir_5444/job/WMTaskSpace/cmsRun1/ALCARECOStreamDtCalib.root',
                                                             'prep_id': 'None',
                                                             'processingStr': 'RECOCOSD_TaskChain_Data_pile_up_test',
                                                             'processingVer': 1,
                                                             'runs': {'160960': [164,
                                                                                 165]},
                                                             'size': 647376,
                                                             'user_dn': None,
                                                             'user_vogroup': 'DEFAULT',
                                                             'user_vorole': 'DEFAULT',
                                                             'validStatus': 'PRODUCTION'}],
                                  'ALCARECOStreamMuAlCalIsolatedMu': [{'InputPFN': '/pool/condor/dir_2661042/glide_qqd8kp/execute/dir_5444/job/WMTaskSpace/cmsRun1/ALCARECOStreamMuAlCalIsolatedMu.root',
                                                                       'OutputPFN': 'root://eoscms.cern.ch//eos/cms/store/unmerged/CMSSW_7_0_0_pre11/Cosmics/ALCARECO/MuAlCalIsolatedMu-RECOCOSD_TaskChain_Data_pile_up_test-v1/00000/9665EB21-08CB-E511-9F4C-02163E017804.root',
                                                                       'StageOutCommand': 'rfcp-CERN',
                                                                       'acquisitionEra': 'CMSSW_7_0_0_pre11',
                                                                       'async_dest': None,
                                                                       'branch_hash': '1569b89a7f6b4a5a6cbeae5b8fccea94',
                                                                       'catalog': '',
                                                                       'checksums': {'adler32': '3379c136',
                                                                                     'cksum': '1828182610'},
                                                                       'configURL': 'https://cmsweb.cern.ch/couchdb;;reqmgr_config_cache;;5f4811e9ccd63d563cd62572350f0db8',
                                                                       'dataset': {'applicationName': 'cmsRun',
                                                                                   'applicationVersion': 'CMSSW_7_0_0_pre11',
                                                                                   'dataTier': 'ALCARECO',
                                                                                   'primaryDataset': 'Cosmics',
                                                                                   'processedDataset': 'CMSSW_7_0_0_pre11-MuAlCalIsolatedMu-RECOCOSD_TaskChain_Data_pile_up_test-v1'},
                                                                       'events': 0,
                                                                       'globalTag': 'GR_R_62_V3::All',
                                                                       'guid': '9665EB21-08CB-E511-9F4C-02163E017804',
                                                                       'input': ['/store/data/Run2011A/Cosmics/RAW/v1/000/160/960/E8099605-8853-E011-A848-0030487A18F2.root'],
                                                                       'inputPath': '/Cosmics/Run2011A-v1/RAW',
                                                                       'inputpfns': ['root://eoscms.cern.ch//eos/cms/store/data/Run2011A/Cosmics/RAW/v1/000/160/960/E8099605-8853-E011-A848-0030487A18F2.root'],
                                                                       'lfn': '/store/unmerged/CMSSW_7_0_0_pre11/Cosmics/ALCARECO/MuAlCalIsolatedMu-RECOCOSD_TaskChain_Data_pile_up_test-v1/00000/9665EB21-08CB-E511-9F4C-02163E017804.root',
                                                                       'location': None,
                                                                       'merged': False,
                                                                       'module_label': 'ALCARECOStreamMuAlCalIsolatedMu',
                                                                       'ouput_module_class': 'PoolOutputModule',
                                                                       'pfn': '/pool/condor/dir_2661042/glide_qqd8kp/execute/dir_5444/job/WMTaskSpace/cmsRun1/ALCARECOStreamMuAlCalIsolatedMu.root',
                                                                       'prep_id': 'None',
                                                                       'processingStr': 'RECOCOSD_TaskChain_Data_pile_up_test',
                                                                       'processingVer': 1,
                                                                       'runs': {'160960': [164,
                                                                                           165]},
                                                                       'size': 665701,
                                                                       'user_dn': None,
                                                                       'user_vogroup': 'DEFAULT',
                                                                       'user_vorole': 'DEFAULT',
                                                                       'validStatus': 'PRODUCTION'}],
                                  'analysis': []}
                       },
                'logArch1': {'analysis': {},
                        'cleanup': {},
                        'errors': [],
                        'input': {},
                        'logs': {},
                        'output': {'analysis': [],
                                   'logArchive': [{'checksums': {'adler32': '6588e920',
                                                                 'cksum': '2315739066'},
                                                   'events': 0,
                                                   'lfn': '/store/unmerged/logs/prod/2016/2/4/sryu_TaskChain_Data_wq_testt_160204_061048_5587/RECOCOSD/0000/0/7d7d41dc-cb02-11e5-833c-02163e00efd5-88-0-logArchive.tar.gz',
                                                   'location': None,
                                                   'merged': False,
                                                   'module_label': 'logArchive',
                                                   'pfn': 'root://eoscms.cern.ch//eos/cms/store/unmerged/logs/prod/2016/2/4/sryu_TaskChain_Data_wq_testt_160204_061048_5587/RECOCOSD/0000/0/7d7d41dc-cb02-11e5-833c-02163e00efd5-88-0-logArchive.tar.gz',
                                                   'runs': {},
                                                   'size': 0}]},
                        'parameters': {},
                        'performance': {'cpu': {},
                                        'memory': {},
                                        'multicore': {},
                                        'storage': {}},
                        'site': 'T2_CH_CERN',
                        'start': 1454569735,
                        'status': 0,
                        'stop': 1454569736},
           'stageOut1': {'analysis': {},
                         'cleanup': {},
                         'errors': [],
                         'input': {},
                         'logs': {},
                         'output': {'analysis': []},
                         'parameters': {},
                         'performance': {'cpu': {},
                                         'memory': {},
                                         'multicore': {},
                                         'storage': {}},
                         'site': 'T2_CH_CERN',
                         'start': 1454569727,
                         'status': 0,
                         'stop': 1454569735}},
 'task': '/sryu_TaskChain_Data_wq_testt_160204_061048_5587/RECOCOSD'}


class FWJRDBAPITest(unittest.TestCase):
    """
    """
    def setUp(self):
        """
        _setUp_
        """
        self.schema = []
        self.couchApps = ["FWJRDump"]
        self.testInit = TestInitCouchApp('FWJRAPITest')
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules = self.schema,
                                useDefault = False)
        dbName = 'fwjrdump_t'
        self.testInit.setupCouch(dbName, *self.couchApps)
        fwjrDBURL = "%s/%s" % (self.testInit.couchUrl, dbName)
        self.fwjrAPI= FWJRDBAPI(fwjrDBURL)
        self.fwjrAPI.defaultStale = {}
        return

    def tearDown(self):
        """
        _tearDown_

        Drop all the WMBS tables.
        """
        self.testInit.tearDownCouch()

    def testFWJRDBAPI(self):

        jobID = 1
        retryCount = 0
        fwjrDocument = {"_id": "%s-%s" % (jobID, retryCount),
                        "jobid": jobID,
                        "retrycount": retryCount,
                        "archivestatus": "ready",
                        "fwjr": SAMPLE_FWJR,
                        "jobtype": "Merge",
                        "type": "fwjr"}
        workflow = SAMPLE_FWJR['task'].split('/')[1]

        self.assertEqual(self.fwjrAPI.couchDB.commitOne(fwjrDocument)[0]['id'], fwjrDocument['_id'])
        self.assertEqual(self.fwjrAPI.getFWJRByArchiveStatus("ready")['rows'][0]['id'], fwjrDocument['_id'])
        self.assertEqual(self.fwjrAPI.isAllFWJRArchived(workflow), False)
        self.fwjrAPI.updateArchiveUploadedStatus(fwjrDocument['_id'])
        self.assertEqual(self.fwjrAPI.getFWJRByArchiveStatus("uploaded")['rows'][0]['id'], fwjrDocument['_id'])
        self.assertEqual(self.fwjrAPI.isAllFWJRArchived(workflow), True)
        self.assertEqual(self.fwjrAPI.getFWJRWithSkippedFiles()['rows'][0]['value']['skippedFiles'], 2)

if __name__ == '__main__':

    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
WMInit_t.py

Created by Dave Evans on 2011-05-20.
Copyright (c) 2011 Fermilab. All rights reserved.
"""
from __future__ import print_function

import os
import threading
import unittest

from WMCore.WMInit import WMInit, getWMBASE


class WMInit_t(unittest.TestCase):
    def testA(self):

        try:
            getWMBASE()
        except:
            self.fail("Error calling WMInit.getWMBASE")

    def testMySQLDatabase(self):
        """
        Testing MySQL basic operations
        """
        dialect = os.environ.get("DIALECT", "MySQL")
        if dialect.lower() == 'oracle':
            # this test can only run for MySQL
            return

        init = WMInit()
        url = os.environ.get("DATABASE")
        sock = os.environ.get("DBSOCK", None)
        init.setDatabaseConnection(url, dialect, sock)

        selectDbName = "SELECT DATABASE() AS dbname"
        destroyDbName = "DROP DATABASE %s"

        try:
            # Initial clear should work
            myThread = threading.currentThread()
            init.clearDatabase()

            # Clear non-existant DB should work
            init.clearDatabase()

            init.setSchema(modules=['WMCore.WMBS'])

            # Drop the database, and then make sure the database gets recreated
            a = myThread.dbi.engine.url.database
            self.assertEqual(myThread.dbi.engine.name, "mysql")
            self.assertTrue(myThread.dbi.engine.url.database in ("wmcore_unittest", "WMCore_unit_test"))
            self.assertEqual(myThread.dbi.engine.url.get_backend_name(), "mysql")
            self.assertEqual(myThread.dbi.engine.url.get_driver_name(), "mysqldb")
            self.assertEqual(myThread.dbi.engine.url.host, "localhost")

            dbName = myThread.dbi.processData(selectDbName)[0].fetchall()[0][0]
            self.assertEqual(dbName, a)
            myThread.dbi.processData(destroyDbName % dbName)
            dbName = myThread.dbi.processData(selectDbName)[0].fetchall()[0][0]
            self.assertEqual(dbName, None)
            init.clearDatabase()
            dbName = myThread.dbi.processData(selectDbName)[0].fetchall()[0][0]
            self.assertEqual(dbName, a)

            init.setSchema(modules=['WMCore.WMBS'])
            myThread.transaction.begin()
            myThread.transaction.processData("SELECT * FROM wmbs_job")
            init.clearDatabase()
            dbName = myThread.dbi.processData(selectDbName)[0].fetchall()[0][0]
            self.assertEqual(dbName, a)
            myThread.transaction.begin()
            init.setSchema(modules=['WMCore.WMBS'])
            myThread.transaction.commit()
        except:
            init.clearDatabase()
            raise
        else:
            init.clearDatabase()

        return

    def testOracleDatabase(self):
        """
        Testing Oracle basic operations
        """
        dialect = os.environ.get("DIALECT", "MySQL")
        if dialect.lower() == 'mysql':
            # this test can only run for Oracle
            return

        init = WMInit()
        url = os.environ.get("DATABASE")
        init.setDatabaseConnection(url, dialect)

        selectDbName = "SELECT ora_database_name FROM DUAL"
        destroyDb = """DECLARE
                 BEGIN

                   execute immediate 'purge recyclebin';

                   -- Tables
                   FOR o IN (SELECT table_name name FROM user_tables) LOOP
                     execute immediate 'drop table ' || o.name || ' cascade constraints';
                   END LOOP;

                   -- Sequences
                   FOR o IN (SELECT sequence_name name FROM user_sequences) LOOP
                     execute immediate 'drop sequence ' || o.name;
                   END LOOP;

                   -- Triggers
                   FOR o IN (SELECT trigger_name name FROM user_triggers) LOOP
                     execute immediate 'drop trigger ' || o.name;
                   END LOOP;

                   -- Synonyms
                   FOR o IN (SELECT synonym_name name FROM user_synonyms) LOOP
                     execute immediate 'drop synonym ' || o.name;
                   END LOOP;

                   -- Functions
                   FOR o IN (SELECT object_name name FROM user_objects WHERE object_type = 'FUNCTION') LOOP
                     execute immediate 'drop function ' || o.name;
                   END LOOP;

                   -- Procedures
                   FOR o IN (SELECT object_name name FROM user_objects WHERE object_type = 'PROCEDURE') LOOP
                     execute immediate 'drop procedure ' || o.name;
                   END LOOP;

                   execute immediate 'purge recyclebin';

                 END;"""

        try:
            # Initial clear should work
            myThread = threading.currentThread()
            init.clearDatabase()

            # Clear non-existant DB should work
            init.clearDatabase()

            init.setSchema(modules=['WMCore.WMBS'])

            # Drop the database, and then make sure the database gets recreated
            a = myThread.dbi.engine.url.database
            self.assertEqual(myThread.dbi.engine.name, "oracle")
            self.assertIsNone(myThread.dbi.engine.url.database)
            self.assertEqual(myThread.dbi.engine.url.get_backend_name(), "oracle")
            self.assertEqual(myThread.dbi.engine.url.get_driver_name(), "cx_oracle")
            self.assertEqual(myThread.dbi.engine.url.host, "INT2R_NOLB")

            dbName = myThread.dbi.processData(selectDbName)[0].fetchall()[0][0]
            self.assertTrue(dbName)
            myThread.transaction.processData("SELECT * FROM wmbs_job")

            init.clearDatabase()
            dbName = myThread.dbi.processData(selectDbName)[0].fetchall()[0][0]
            self.assertTrue(dbName)

            myThread.dbi.processData(destroyDb)

            init.setSchema(modules=['WMCore.WMBS'])
            myThread.transaction.begin()
            myThread.transaction.processData("SELECT * FROM wmbs_job")
            init.clearDatabase()
            dbName = myThread.dbi.processData(selectDbName)[0].fetchall()[0][0]
            self.assertTrue(dbName)
            myThread.transaction.begin()
            init.setSchema(modules=['WMCore.WMBS'])
            myThread.transaction.commit()
        except:
            init.clearDatabase()
            raise
        else:
            init.clearDatabase()



if __name__ == '__main__':
    unittest.main()
